@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,13 @@
 # limitations under the License.
 #
 import os
-import mock
+
+# try/except added for compatibility with python < 3.8
+try:
+    from unittest import mock
+    from unittest.mock import AsyncMock
+except ImportError:
+    import mock
 
 import grpc
 from grpc.experimental import aio
@@ -90,20 +96,26 @@ def test__get_default_mtls_endpoint():
 
 
 @pytest.mark.parametrize(
-    "client_class", [SystemPolicyV1Client, SystemPolicyV1AsyncClient,]
+    "client_class,transport_name",
+    [
+        (SystemPolicyV1Client, "grpc"),
+        (SystemPolicyV1AsyncClient, "grpc_asyncio"),
+    ],
 )
-def test_system_policy_v1_client_from_service_account_info(client_class):
+def test_system_policy_v1_client_from_service_account_info(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_info"
     ) as factory:
         factory.return_value = creds
         info = {"valid": True}
-        client = client_class.from_service_account_info(info)
+        client = client_class.from_service_account_info(info, transport=transport_name)
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "binaryauthorization.googleapis.com:443"
+        assert client.transport._host == ("binaryauthorization.googleapis.com:443")
 
 
 @pytest.mark.parametrize(
@@ -132,23 +144,33 @@ def test_system_policy_v1_client_service_account_always_use_jwt(
 
 
 @pytest.mark.parametrize(
-    "client_class", [SystemPolicyV1Client, SystemPolicyV1AsyncClient,]
+    "client_class,transport_name",
+    [
+        (SystemPolicyV1Client, "grpc"),
+        (SystemPolicyV1AsyncClient, "grpc_asyncio"),
+    ],
 )
-def test_system_policy_v1_client_from_service_account_file(client_class):
+def test_system_policy_v1_client_from_service_account_file(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_file"
     ) as factory:
         factory.return_value = creds
-        client = client_class.from_service_account_file("dummy/file/path.json")
+        client = client_class.from_service_account_file(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        client = client_class.from_service_account_json("dummy/file/path.json")
+        client = client_class.from_service_account_json(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "binaryauthorization.googleapis.com:443"
+        assert client.transport._host == ("binaryauthorization.googleapis.com:443")
 
 
 def test_system_policy_v1_client_get_transport_class():
@@ -496,7 +518,9 @@ def test_system_policy_v1_client_client_options_scopes(
     client_class, transport_class, transport_name
 ):
     # Check the case scopes are provided.
-    options = client_options.ClientOptions(scopes=["1", "2"],)
+    options = client_options.ClientOptions(
+        scopes=["1", "2"],
+    )
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
         client = client_class(client_options=options, transport=transport_name)
@@ -636,10 +660,17 @@ def test_system_policy_v1_client_create_channel_credentials_file(
         )
 
 
-@pytest.mark.parametrize("request_type", [service.GetSystemPolicyRequest, dict,])
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        service.GetSystemPolicyRequest,
+        dict,
+    ],
+)
 def test_get_system_policy(request_type, transport: str = "grpc"):
     client = SystemPolicyV1Client(
-        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -677,7 +708,8 @@ def test_get_system_policy_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
     client = SystemPolicyV1Client(
-        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="grpc",
     )
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -695,7 +727,8 @@ async def test_get_system_policy_async(
     transport: str = "grpc_asyncio", request_type=service.GetSystemPolicyRequest
 ):
     client = SystemPolicyV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -737,13 +770,15 @@ async def test_get_system_policy_async_from_dict():
 
 
 def test_get_system_policy_field_headers():
-    client = SystemPolicyV1Client(credentials=ga_credentials.AnonymousCredentials(),)
+    client = SystemPolicyV1Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
     # a field header. Set these to a non-empty value.
     request = service.GetSystemPolicyRequest()
 
-    request.name = "name/value"
+    request.name = "name_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -759,7 +794,10 @@ def test_get_system_policy_field_headers():
 
     # Establish that the field header was sent.
     _, _, kw = call.mock_calls[0]
-    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]
+    assert (
+        "x-goog-request-params",
+        "name=name_value",
+    ) in kw["metadata"]
 
 
 @pytest.mark.asyncio
@@ -772,7 +810,7 @@ async def test_get_system_policy_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = service.GetSystemPolicyRequest()
 
-    request.name = "name/value"
+    request.name = "name_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -788,11 +826,16 @@ async def test_get_system_policy_field_headers_async():
 
     # Establish that the field header was sent.
     _, _, kw = call.mock_calls[0]
-    assert ("x-goog-request-params", "name=name/value",) in kw["metadata"]
+    assert (
+        "x-goog-request-params",
+        "name=name_value",
+    ) in kw["metadata"]
 
 
 def test_get_system_policy_flattened():
-    client = SystemPolicyV1Client(credentials=ga_credentials.AnonymousCredentials(),)
+    client = SystemPolicyV1Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -802,7 +845,9 @@ def test_get_system_policy_flattened():
         call.return_value = resources.Policy()
         # Call the method with a truthy value for each flattened field,
         # using the keyword arguments to the method.
-        client.get_system_policy(name="name_value",)
+        client.get_system_policy(
+            name="name_value",
+        )
 
         # Establish that the underlying call was made with the expected
         # request object values.
@@ -814,13 +859,16 @@ def test_get_system_policy_flattened():
 
 
 def test_get_system_policy_flattened_error():
-    client = SystemPolicyV1Client(credentials=ga_credentials.AnonymousCredentials(),)
+    client = SystemPolicyV1Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
 
     # Attempting to call a method with both a request object and flattened
     # fields is an error.
     with pytest.raises(ValueError):
         client.get_system_policy(
-            service.GetSystemPolicyRequest(), name="name_value",
+            service.GetSystemPolicyRequest(),
+            name="name_value",
         )
 
 
@@ -840,7 +888,9 @@ async def test_get_system_policy_flattened_async():
         call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(resources.Policy())
         # Call the method with a truthy value for each flattened field,
         # using the keyword arguments to the method.
-        response = await client.get_system_policy(name="name_value",)
+        response = await client.get_system_policy(
+            name="name_value",
+        )
 
         # Establish that the underlying call was made with the expected
         # request object values.
@@ -861,7 +911,8 @@ async def test_get_system_policy_flattened_error_async():
     # fields is an error.
     with pytest.raises(ValueError):
         await client.get_system_policy(
-            service.GetSystemPolicyRequest(), name="name_value",
+            service.GetSystemPolicyRequest(),
+            name="name_value",
         )
 
 
@@ -872,7 +923,8 @@ def test_credentials_transport_error():
     )
     with pytest.raises(ValueError):
         client = SystemPolicyV1Client(
-            credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+            credentials=ga_credentials.AnonymousCredentials(),
+            transport=transport,
         )
 
     # It is an error to provide a credentials file and a transport instance.
@@ -892,7 +944,10 @@ def test_credentials_transport_error():
     options = client_options.ClientOptions()
     options.api_key = "api_key"
     with pytest.raises(ValueError):
-        client = SystemPolicyV1Client(client_options=options, transport=transport,)
+        client = SystemPolicyV1Client(
+            client_options=options,
+            transport=transport,
+        )
 
     # It is an error to provide an api_key and a credential.
     options = mock.Mock()
@@ -908,7 +963,8 @@ def test_credentials_transport_error():
     )
     with pytest.raises(ValueError):
         client = SystemPolicyV1Client(
-            client_options={"scopes": ["1", "2"]}, transport=transport,
+            client_options={"scopes": ["1", "2"]},
+            transport=transport,
         )
 
 
@@ -951,10 +1007,28 @@ def test_transport_adc(transport_class):
         adc.assert_called_once()
 
 
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+    ],
+)
+def test_transport_kind(transport_name):
+    transport = SystemPolicyV1Client.get_transport_class(transport_name)(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    assert transport.kind == transport_name
+
+
 def test_transport_grpc_default():
     # A client should use the gRPC transport by default.
-    client = SystemPolicyV1Client(credentials=ga_credentials.AnonymousCredentials(),)
-    assert isinstance(client.transport, transports.SystemPolicyV1GrpcTransport,)
+    client = SystemPolicyV1Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    assert isinstance(
+        client.transport,
+        transports.SystemPolicyV1GrpcTransport,
+    )
 
 
 def test_system_policy_v1_base_transport_error():
@@ -986,6 +1060,14 @@ def test_system_policy_v1_base_transport():
     with pytest.raises(NotImplementedError):
         transport.close()
 
+    # Catch all for all remaining methods and properties
+    remainder = [
+        "kind",
+    ]
+    for r in remainder:
+        with pytest.raises(NotImplementedError):
+            getattr(transport, r)()
+
 
 def test_system_policy_v1_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
@@ -997,7 +1079,8 @@ def test_system_policy_v1_base_transport_with_credentials_file():
         Transport.return_value = None
         load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport = transports.SystemPolicyV1Transport(
-            credentials_file="credentials.json", quota_project_id="octopus",
+            credentials_file="credentials.json",
+            quota_project_id="octopus",
         )
         load_creds.assert_called_once_with(
             "credentials.json",
@@ -1130,24 +1213,40 @@ def test_system_policy_v1_grpc_transport_client_cert_source_for_mtls(transport_c
             )
 
 
-def test_system_policy_v1_host_no_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_system_policy_v1_host_no_port(transport_name):
     client = SystemPolicyV1Client(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="binaryauthorization.googleapis.com"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "binaryauthorization.googleapis.com:443"
+    assert client.transport._host == ("binaryauthorization.googleapis.com:443")
 
 
-def test_system_policy_v1_host_with_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_system_policy_v1_host_with_port(transport_name):
     client = SystemPolicyV1Client(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="binaryauthorization.googleapis.com:8000"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "binaryauthorization.googleapis.com:8000"
+    assert client.transport._host == ("binaryauthorization.googleapis.com:8000")
 
 
 def test_system_policy_v1_grpc_transport_channel():
@@ -1155,7 +1254,8 @@ def test_system_policy_v1_grpc_transport_channel():
 
     # Check that channel is used if provided.
     transport = transports.SystemPolicyV1GrpcTransport(
-        host="squid.clam.whelk", channel=channel,
+        host="squid.clam.whelk",
+        channel=channel,
     )
     assert transport.grpc_channel == channel
     assert transport._host == "squid.clam.whelk:443"
@@ -1167,7 +1267,8 @@ def test_system_policy_v1_grpc_asyncio_transport_channel():
 
     # Check that channel is used if provided.
     transport = transports.SystemPolicyV1GrpcAsyncIOTransport(
-        host="squid.clam.whelk", channel=channel,
+        host="squid.clam.whelk",
+        channel=channel,
     )
     assert transport.grpc_channel == channel
     assert transport._host == "squid.clam.whelk:443"
@@ -1276,7 +1377,9 @@ def test_system_policy_v1_transport_channel_mtls_with_adc(transport_class):
 
 def test_policy_path():
     project = "squid"
-    expected = "projects/{project}/policy".format(project=project,)
+    expected = "projects/{project}/policy".format(
+        project=project,
+    )
     actual = SystemPolicyV1Client.policy_path(project)
     assert expected == actual
 
@@ -1314,7 +1417,9 @@ def test_parse_common_billing_account_path():
 
 def test_common_folder_path():
     folder = "oyster"
-    expected = "folders/{folder}".format(folder=folder,)
+    expected = "folders/{folder}".format(
+        folder=folder,
+    )
     actual = SystemPolicyV1Client.common_folder_path(folder)
     assert expected == actual
 
@@ -1332,7 +1437,9 @@ def test_parse_common_folder_path():
 
 def test_common_organization_path():
     organization = "cuttlefish"
-    expected = "organizations/{organization}".format(organization=organization,)
+    expected = "organizations/{organization}".format(
+        organization=organization,
+    )
     actual = SystemPolicyV1Client.common_organization_path(organization)
     assert expected == actual
 
@@ -1350,7 +1457,9 @@ def test_parse_common_organization_path():
 
 def test_common_project_path():
     project = "winkle"
-    expected = "projects/{project}".format(project=project,)
+    expected = "projects/{project}".format(
+        project=project,
+    )
     actual = SystemPolicyV1Client.common_project_path(project)
     assert expected == actual
 
@@ -1370,7 +1479,8 @@ def test_common_location_path():
     project = "scallop"
     location = "abalone"
     expected = "projects/{project}/locations/{location}".format(
-        project=project, location=location,
+        project=project,
+        location=location,
     )
     actual = SystemPolicyV1Client.common_location_path(project, location)
     assert expected == actual
@@ -1395,7 +1505,8 @@ def test_client_with_default_client_info():
         transports.SystemPolicyV1Transport, "_prep_wrapped_messages"
     ) as prep:
         client = SystemPolicyV1Client(
-            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(),
+            client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
 
@@ -1404,7 +1515,8 @@ def test_client_with_default_client_info():
     ) as prep:
         transport_class = SystemPolicyV1Client.get_transport_class()
         transport = transport_class(
-            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(),
+            client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
 
@@ -1412,7 +1524,8 @@ def test_client_with_default_client_info():
 @pytest.mark.asyncio
 async def test_transport_close_async():
     client = SystemPolicyV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(), transport="grpc_asyncio",
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="grpc_asyncio",
     )
     with mock.patch.object(
         type(getattr(client.transport, "grpc_channel")), "close"
