@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,13 @@
 # limitations under the License.
 #
 import os
-import mock
+
+# try/except added for compatibility with python < 3.8
+try:
+    from unittest import mock
+    from unittest.mock import AsyncMock
+except ImportError:
+    import mock
 
 import grpc
 from grpc.experimental import aio
@@ -91,20 +97,26 @@ def test__get_default_mtls_endpoint():
 
 
 @pytest.mark.parametrize(
-    "client_class", [ValidationHelperV1Client, ValidationHelperV1AsyncClient,]
+    "client_class,transport_name",
+    [
+        (ValidationHelperV1Client, "grpc"),
+        (ValidationHelperV1AsyncClient, "grpc_asyncio"),
+    ],
 )
-def test_validation_helper_v1_client_from_service_account_info(client_class):
+def test_validation_helper_v1_client_from_service_account_info(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_info"
     ) as factory:
         factory.return_value = creds
         info = {"valid": True}
-        client = client_class.from_service_account_info(info)
+        client = client_class.from_service_account_info(info, transport=transport_name)
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "binaryauthorization.googleapis.com:443"
+        assert client.transport._host == ("binaryauthorization.googleapis.com:443")
 
 
 @pytest.mark.parametrize(
@@ -133,23 +145,33 @@ def test_validation_helper_v1_client_service_account_always_use_jwt(
 
 
 @pytest.mark.parametrize(
-    "client_class", [ValidationHelperV1Client, ValidationHelperV1AsyncClient,]
+    "client_class,transport_name",
+    [
+        (ValidationHelperV1Client, "grpc"),
+        (ValidationHelperV1AsyncClient, "grpc_asyncio"),
+    ],
 )
-def test_validation_helper_v1_client_from_service_account_file(client_class):
+def test_validation_helper_v1_client_from_service_account_file(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_file"
     ) as factory:
         factory.return_value = creds
-        client = client_class.from_service_account_file("dummy/file/path.json")
+        client = client_class.from_service_account_file(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        client = client_class.from_service_account_json("dummy/file/path.json")
+        client = client_class.from_service_account_json(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "binaryauthorization.googleapis.com:443"
+        assert client.transport._host == ("binaryauthorization.googleapis.com:443")
 
 
 def test_validation_helper_v1_client_get_transport_class():
@@ -507,7 +529,9 @@ def test_validation_helper_v1_client_client_options_scopes(
     client_class, transport_class, transport_name
 ):
     # Check the case scopes are provided.
-    options = client_options.ClientOptions(scopes=["1", "2"],)
+    options = client_options.ClientOptions(
+        scopes=["1", "2"],
+    )
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
         client = client_class(client_options=options, transport=transport_name)
@@ -648,11 +672,16 @@ def test_validation_helper_v1_client_create_channel_credentials_file(
 
 
 @pytest.mark.parametrize(
-    "request_type", [service.ValidateAttestationOccurrenceRequest, dict,]
+    "request_type",
+    [
+        service.ValidateAttestationOccurrenceRequest,
+        dict,
+    ],
 )
 def test_validate_attestation_occurrence(request_type, transport: str = "grpc"):
     client = ValidationHelperV1Client(
-        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -687,7 +716,8 @@ def test_validate_attestation_occurrence_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
     client = ValidationHelperV1Client(
-        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="grpc",
     )
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -706,7 +736,8 @@ async def test_validate_attestation_occurrence_async(
     request_type=service.ValidateAttestationOccurrenceRequest,
 ):
     client = ValidationHelperV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -753,7 +784,7 @@ def test_validate_attestation_occurrence_field_headers():
     # a field header. Set these to a non-empty value.
     request = service.ValidateAttestationOccurrenceRequest()
 
-    request.attestor = "attestor/value"
+    request.attestor = "attestor_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -769,7 +800,10 @@ def test_validate_attestation_occurrence_field_headers():
 
     # Establish that the field header was sent.
     _, _, kw = call.mock_calls[0]
-    assert ("x-goog-request-params", "attestor=attestor/value",) in kw["metadata"]
+    assert (
+        "x-goog-request-params",
+        "attestor=attestor_value",
+    ) in kw["metadata"]
 
 
 @pytest.mark.asyncio
@@ -782,7 +816,7 @@ async def test_validate_attestation_occurrence_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = service.ValidateAttestationOccurrenceRequest()
 
-    request.attestor = "attestor/value"
+    request.attestor = "attestor_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -800,7 +834,10 @@ async def test_validate_attestation_occurrence_field_headers_async():
 
     # Establish that the field header was sent.
     _, _, kw = call.mock_calls[0]
-    assert ("x-goog-request-params", "attestor=attestor/value",) in kw["metadata"]
+    assert (
+        "x-goog-request-params",
+        "attestor=attestor_value",
+    ) in kw["metadata"]
 
 
 def test_credentials_transport_error():
@@ -810,7 +847,8 @@ def test_credentials_transport_error():
     )
     with pytest.raises(ValueError):
         client = ValidationHelperV1Client(
-            credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+            credentials=ga_credentials.AnonymousCredentials(),
+            transport=transport,
         )
 
     # It is an error to provide a credentials file and a transport instance.
@@ -830,7 +868,10 @@ def test_credentials_transport_error():
     options = client_options.ClientOptions()
     options.api_key = "api_key"
     with pytest.raises(ValueError):
-        client = ValidationHelperV1Client(client_options=options, transport=transport,)
+        client = ValidationHelperV1Client(
+            client_options=options,
+            transport=transport,
+        )
 
     # It is an error to provide an api_key and a credential.
     options = mock.Mock()
@@ -846,7 +887,8 @@ def test_credentials_transport_error():
     )
     with pytest.raises(ValueError):
         client = ValidationHelperV1Client(
-            client_options={"scopes": ["1", "2"]}, transport=transport,
+            client_options={"scopes": ["1", "2"]},
+            transport=transport,
         )
 
 
@@ -889,12 +931,28 @@ def test_transport_adc(transport_class):
         adc.assert_called_once()
 
 
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+    ],
+)
+def test_transport_kind(transport_name):
+    transport = ValidationHelperV1Client.get_transport_class(transport_name)(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    assert transport.kind == transport_name
+
+
 def test_transport_grpc_default():
     # A client should use the gRPC transport by default.
     client = ValidationHelperV1Client(
         credentials=ga_credentials.AnonymousCredentials(),
     )
-    assert isinstance(client.transport, transports.ValidationHelperV1GrpcTransport,)
+    assert isinstance(
+        client.transport,
+        transports.ValidationHelperV1GrpcTransport,
+    )
 
 
 def test_validation_helper_v1_base_transport_error():
@@ -926,6 +984,14 @@ def test_validation_helper_v1_base_transport():
     with pytest.raises(NotImplementedError):
         transport.close()
 
+    # Catch all for all remaining methods and properties
+    remainder = [
+        "kind",
+    ]
+    for r in remainder:
+        with pytest.raises(NotImplementedError):
+            getattr(transport, r)()
+
 
 def test_validation_helper_v1_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
@@ -937,7 +1003,8 @@ def test_validation_helper_v1_base_transport_with_credentials_file():
         Transport.return_value = None
         load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport = transports.ValidationHelperV1Transport(
-            credentials_file="credentials.json", quota_project_id="octopus",
+            credentials_file="credentials.json",
+            quota_project_id="octopus",
         )
         load_creds.assert_called_once_with(
             "credentials.json",
@@ -1072,24 +1139,40 @@ def test_validation_helper_v1_grpc_transport_client_cert_source_for_mtls(
             )
 
 
-def test_validation_helper_v1_host_no_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_validation_helper_v1_host_no_port(transport_name):
     client = ValidationHelperV1Client(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="binaryauthorization.googleapis.com"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "binaryauthorization.googleapis.com:443"
+    assert client.transport._host == ("binaryauthorization.googleapis.com:443")
 
 
-def test_validation_helper_v1_host_with_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_validation_helper_v1_host_with_port(transport_name):
     client = ValidationHelperV1Client(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="binaryauthorization.googleapis.com:8000"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "binaryauthorization.googleapis.com:8000"
+    assert client.transport._host == ("binaryauthorization.googleapis.com:8000")
 
 
 def test_validation_helper_v1_grpc_transport_channel():
@@ -1097,7 +1180,8 @@ def test_validation_helper_v1_grpc_transport_channel():
 
     # Check that channel is used if provided.
     transport = transports.ValidationHelperV1GrpcTransport(
-        host="squid.clam.whelk", channel=channel,
+        host="squid.clam.whelk",
+        channel=channel,
     )
     assert transport.grpc_channel == channel
     assert transport._host == "squid.clam.whelk:443"
@@ -1109,7 +1193,8 @@ def test_validation_helper_v1_grpc_asyncio_transport_channel():
 
     # Check that channel is used if provided.
     transport = transports.ValidationHelperV1GrpcAsyncIOTransport(
-        host="squid.clam.whelk", channel=channel,
+        host="squid.clam.whelk",
+        channel=channel,
     )
     assert transport.grpc_channel == channel
     assert transport._host == "squid.clam.whelk:443"
@@ -1238,7 +1323,9 @@ def test_parse_common_billing_account_path():
 
 def test_common_folder_path():
     folder = "whelk"
-    expected = "folders/{folder}".format(folder=folder,)
+    expected = "folders/{folder}".format(
+        folder=folder,
+    )
     actual = ValidationHelperV1Client.common_folder_path(folder)
     assert expected == actual
 
@@ -1256,7 +1343,9 @@ def test_parse_common_folder_path():
 
 def test_common_organization_path():
     organization = "oyster"
-    expected = "organizations/{organization}".format(organization=organization,)
+    expected = "organizations/{organization}".format(
+        organization=organization,
+    )
     actual = ValidationHelperV1Client.common_organization_path(organization)
     assert expected == actual
 
@@ -1274,7 +1363,9 @@ def test_parse_common_organization_path():
 
 def test_common_project_path():
     project = "cuttlefish"
-    expected = "projects/{project}".format(project=project,)
+    expected = "projects/{project}".format(
+        project=project,
+    )
     actual = ValidationHelperV1Client.common_project_path(project)
     assert expected == actual
 
@@ -1294,7 +1385,8 @@ def test_common_location_path():
     project = "winkle"
     location = "nautilus"
     expected = "projects/{project}/locations/{location}".format(
-        project=project, location=location,
+        project=project,
+        location=location,
     )
     actual = ValidationHelperV1Client.common_location_path(project, location)
     assert expected == actual
@@ -1319,7 +1411,8 @@ def test_client_with_default_client_info():
         transports.ValidationHelperV1Transport, "_prep_wrapped_messages"
     ) as prep:
         client = ValidationHelperV1Client(
-            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(),
+            client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
 
@@ -1328,7 +1421,8 @@ def test_client_with_default_client_info():
     ) as prep:
         transport_class = ValidationHelperV1Client.get_transport_class()
         transport = transport_class(
-            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(),
+            client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
 
@@ -1336,7 +1430,8 @@ def test_client_with_default_client_info():
 @pytest.mark.asyncio
 async def test_transport_close_async():
     client = ValidationHelperV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(), transport="grpc_asyncio",
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="grpc_asyncio",
     )
     with mock.patch.object(
         type(getattr(client.transport, "grpc_channel")), "close"
